import * as vscode from 'vscode';
import axios from 'axios';
import { exec } from 'child_process';

// Funktion zum Abrufen des Gitea Repositories und Erstellen eines Pull Requests
async function createGiteaPullRequest() {
    const instanceUrl = vscode.workspace.getConfiguration().get<string>('gitea.instanceUrl');
    const token = vscode.workspace.getConfiguration().get<string>('gitea.personalAccessToken');

    if (!instanceUrl || !token) {
        vscode.window.showErrorMessage('Gitea URL oder Token ist nicht konfiguriert.');
        return;
    }

    // Aktuellen Git-Ordner abrufen
    const currentWorkspaceFolder = vscode.workspace.workspaceFolders?.[0]?.uri?.fsPath;
    if (!currentWorkspaceFolder) {
        vscode.window.showErrorMessage('Kein gültiges Git-Repository im aktuellen Workspace gefunden.');
        return;
    }

    // Git-Repository-Informationen abrufen
    exec(`git config --get remote.origin.url`, { cwd: currentWorkspaceFolder }, async (error, stdout, stderr) => {
        if (error || !stdout) {
            vscode.window.showErrorMessage('Konnte die Git-Remote-URL nicht abrufen.');
            return;
        }

        let repoUrl = stdout.trim();

        // URL umwandeln, wenn sie im SSH-Format vorliegt (git@gitea.com:owner/repo.git)
        if (repoUrl.startsWith('git@')) {
            // SSH-URL in HTTP(S)-URL umwandeln
            const parts = repoUrl.split(':');
            const domain = parts[0].replace('git@', '');
            const path = parts[1].replace('.git', '');
            repoUrl = `https://${domain}/${path}`;
        } else {
            // Falls es eine HTTPS-URL ist, einfach .git entfernen
            repoUrl = repoUrl.replace('.git', '');
        }

        // URL für Pull Request Seite generieren
        const pullRequestUrl = `${repoUrl}/pulls/new`;

        try {
            // Verwende die VSCode API, um die URL im Browser zu öffnen
            await vscode.env.openExternal(vscode.Uri.parse(pullRequestUrl));
            vscode.window.showInformationMessage('Pull-Request-Seite im Browser geöffnet.');
        } catch (err: unknown) {
            vscode.window.showErrorMessage('Fehler beim Öffnen der Pull-Request-Seite.');
            console.error('Fehler beim Öffnen des Browsers:', err);
        }
    });
}

// Funktion zum Hinzufügen des Statusbar-Icons
function addStatusBarIcon() {
    const statusBar = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Left);
    statusBar.text = `$(git-pull-request) Create PR`;
    statusBar.tooltip = 'Create a Pull Request in Gitea';
    statusBar.command = 'gitea.createPullRequest';
    statusBar.show();
    return statusBar;
}

// Funktion zur Authentifizierung bei Gitea mit dem Personal Access Token (PAT)
async function authenticateGitea() {
    const instanceUrl = vscode.workspace.getConfiguration().get<string>('gitea.instanceUrl');
    const token = vscode.workspace.getConfiguration().get<string>('gitea.personalAccessToken');

    if (!instanceUrl || !token) {
        vscode.window.showErrorMessage('Gitea URL oder Token ist nicht konfiguriert.');
        return;
    }

    try {
        const response = await axios.get(`${instanceUrl}/api/v1/user`, {
            headers: {
                'Authorization': `token ${token}`
            }
        });

        if (response.status === 200) {
            vscode.window.showInformationMessage(`Authentifizierung erfolgreich: ${response.data.username}`);
        }
    } catch (error) {
        vscode.window.showErrorMessage('Authentifizierung fehlgeschlagen.');
        console.error(error);
    }
}

// Funktion zur Konfiguration der Gitea-Instanz und des Tokens über die Command Palette
async function configureGitea() {
    const instanceUrl = await vscode.window.showInputBox({
        prompt: "Gitea Instanz-URL eingeben",
        placeHolder: "https://your-gitea-instance.com"
    });

    const token = await vscode.window.showInputBox({
        prompt: "Gitea Personal Access Token eingeben",
        placeHolder: "token",
        password: true
    });

    if (instanceUrl && token) {
        await vscode.workspace.getConfiguration().update('gitea.instanceUrl', instanceUrl, vscode.ConfigurationTarget.Global);
        await vscode.workspace.getConfiguration().update('gitea.personalAccessToken', token, vscode.ConfigurationTarget.Global);
        vscode.window.showInformationMessage('Gitea Konfiguration aktualisiert.');
    }
}

// Funktion zum Abrufen der Repositories des Benutzers von Gitea
async function getGiteaRepositories(): Promise<any[]> {
    const instanceUrl = vscode.workspace.getConfiguration().get<string>('gitea.instanceUrl');
    const token = vscode.workspace.getConfiguration().get<string>('gitea.personalAccessToken');

    if (!instanceUrl || !token) {
        vscode.window.showErrorMessage('Gitea URL oder Token ist nicht konfiguriert.');
        return [];
    }

    try {
        const response = await axios.get(`${instanceUrl}/api/v1/user/repos`, {
            headers: {
                'Authorization': `token ${token}`
            }
        });

        if (response.status === 200) {
            return response.data;
        } else {
            vscode.window.showErrorMessage('Fehler beim Abrufen der Repositories.');
            return [];
        }
    } catch (error) {
        vscode.window.showErrorMessage('Fehler bei der Verbindung mit der Gitea API.');
        console.error(error);
        return [];
    }
}

// Funktion zum Klonen eines Repositories via SSH
async function cloneGiteaRepository() {
    const repos = await getGiteaRepositories();

    if (repos.length === 0) {
        vscode.window.showInformationMessage('Keine Repositories gefunden.');
        return;
    }

    const repoNames = repos.map(repo => repo.full_name);
    const selectedRepo = await vscode.window.showQuickPick(repoNames, {
        placeHolder: 'Wähle ein Repository zum Klonen aus'
    });

    if (!selectedRepo) {
        vscode.window.showInformationMessage('Kein Repository ausgewählt.');
        return;
    }

    const repo = repos.find(r => r.full_name === selectedRepo);

    if (repo && repo.ssh_url) {
        const folderUri = await vscode.window.showOpenDialog({
            canSelectFiles: false,
            canSelectFolders: true,
            canSelectMany: false,
            openLabel: 'Ordner zum Klonen auswählen'
        });

        if (folderUri && folderUri[0]) {
            const folderPath = folderUri[0].fsPath;

            vscode.window.withProgress({
                location: vscode.ProgressLocation.Notification,
                title: `Cloning ${selectedRepo}`,
                cancellable: false
            }, async (progress, token) => {
                progress.report({ message: 'Klonvorgang läuft...', increment: 0 });

                return new Promise<void>((resolve, reject) => {
                    exec(`git clone ${repo.ssh_url} ${folderPath}`, (error, stdout, stderr) => {
                        if (error) {
                            vscode.window.showErrorMessage('Fehler beim Klonen des Repositories.');
                            console.error(stderr);
                            reject(error);
                        } else {
                            progress.report({ message: 'Repository geklont.', increment: 100 });
                            vscode.window.showInformationMessage(`Repository ${selectedRepo} erfolgreich geklont.`);

                            // Öffne das geklonte Repository im VSCode
                            try {
                                vscode.commands.executeCommand('vscode.openFolder', vscode.Uri.file(folderPath), true)
                                    .then(() => resolve());
                            } catch (err: unknown) {
                                vscode.window.showErrorMessage('Fehler beim Öffnen des geklonten Repositories.');
                                console.error(err);
                                reject(err);
                            }
                        }
                    });
                });
            });
        } else {
            vscode.window.showInformationMessage('Kein Zielordner ausgewählt.');
        }
    } else {
        vscode.window.showErrorMessage('Konnte die SSH-Klon-URL nicht finden.');
    }
}

// Aktivierungsfunktion des Plugins
export function activate(context: vscode.ExtensionContext) {
    // Registriert den Befehl zur Authentifizierung
    let authCommand = vscode.commands.registerCommand('gitea.authenticate', authenticateGitea);
    context.subscriptions.push(authCommand);

    // Registriert den Befehl zur Konfiguration
    let configCommand = vscode.commands.registerCommand('gitea.configure', configureGitea);
    context.subscriptions.push(configCommand);

	// Befehl zum Klonen eines Repositories
    let cloneCommand = vscode.commands.registerCommand('gitea.cloneRepository', cloneGiteaRepository);
    context.subscriptions.push(cloneCommand);

	// Registriere den Befehl zum Erstellen eines Pull Requests
    let pullRequestCommand = vscode.commands.registerCommand('gitea.createPullRequest', createGiteaPullRequest);
    context.subscriptions.push(pullRequestCommand);

    // Statusbar-Icon hinzufügen
    const statusBar = addStatusBarIcon();
    context.subscriptions.push(statusBar);
}

// Deaktivierungsfunktion des Plugins
export function deactivate() {}
